""" This file initialize the performance metrics and calculate output results."""

class TableResults:
    """
    Output final table evaluation results.
    """
    def __init__(self, metrics, labels):
        self._metrics = metrics
        self._labels = labels
        
    def initMetrics(self):
        """ Metrics initializer

        @remarks This function initialize performance metrics dictionary and specific error categories.
        
        """
        # Arguments for calculating precision and recall.
        self._metrics.setdefault('true', 0)
        self._metrics.setdefault('missed', 0)
        self._metrics.setdefault('acceptable', 0)
        self._metrics.setdefault('fake but acceptable', 0)
        self._metrics.setdefault('fake and unacceptable', 0)
        
        # Error labels and amount of tables hitting each label.
        self._labels.setdefault('fake_matrix', 0)
        self._labels.setdefault('fake_list', 0)
        self._labels.setdefault('fake_figure', 0)
        self._labels.setdefault('fake_otherObj', 0)
        self._labels.setdefault('missed', 0)
        self._labels.setdefault('splitted_horizontal', 0)
        self._labels.setdefault('splitted_vertical', 0)
        self._labels.setdefault('merged', 0)
        self._labels.setdefault('reduced', 0)
        self._labels.setdefault('amplified_tabaccessory', 0)
        self._labels.setdefault('amplified_matrix', 0)
        self._labels.setdefault('amplified_otherObj', 0)


    def output(self):
        """ Output results

        @remarks This function calculates final precision and recall according to the metrics dictionary.
        
        """
        print '\n-----Error labels and amount of tables-----'
        for label, value in self._labels.items():
            print '%s : %d' % (label, value)
            
        print '\n----Arguments for calculating P&R-----'
        for metric, value in self._metrics.items():
            print '%s : %d' % (metric, value)

        print '\n----Precision and Recall-----'
        A = self._metrics['true']
        B = self._metrics['missed']
        C = self._metrics['acceptable']
        D = self._metrics['fake but acceptable']
        E = self._metrics['fake and unacceptable']
        P = R = 0
        if ((A+D+E-B) != 0):
            P= C / float(A+D+E-B)
        if ((A+D) != 0):
            R= C / float(A+D)
            
        print 'Precision: %.2f\n' \
              'Recall: %.2f\n'    \
              % (P, R)
