# coding=utf-8
import sys
import os
import string

""" This file is the entrance to the evaluation methods."""

rootpath = os.path.abspath('../')
sys.path.append(rootpath)

from Common.Labels import *
from Common.Page import Page
from Common.PageReader import PageReader
from TableEvaluator.TableResults import *
from TableEvaluator.TableEvaluator import *


def main():
    """ Entrance function.

    @remarks The main function gets the system arguments, which direct to   \n
             the folders containing the ground-truth and obtained results.  \n
             Table evaluator is also called here.                           \n
    """  
    reload(sys)
    sys.setdefaultencoding(u'utf-8')
    filesys_encoding = sys.getfilesystemencoding()
    stdin_encoding = sys.stdin.encoding
    
    gtdir = sys.argv[1]
    obdir = sys.argv[2]   
    if stdin_encoding:
        gtdir = gtdir.decode(stdin_encoding)
        obdir = obdir.decode(stdin_encoding)

    gtdir = gtdir.encode(filesys_encoding)
    obdir = obdir.encode(filesys_encoding)

    evalTable(gtdir, obdir)

  
def evalTable(gtdir, obdir):
    """ Table Evaluation caller.

    @remarks This function initialize the performance metrics,               \n
             read page content of the pairs of xml files to be compared,     \n
             and give those data to TableEvaluator class.                    \n
             Finally, it outputs the results in command line.                \n
             The corresponding ground-truth file and obtained results file should share the same name.      \n
    
    @param  gtdir  directory of the ground-truth files                  \n
    @param  obdir  directory of the algorithm obtained results files    \n
    """
    reader = PageReader()
    metrics = {}
    lables = {}
    tRes = TableResults(metrics, lables)
    tRes.initMetrics()  
    files = os.listdir(gtdir)
    
    # Assume gt and ob have identical file names.
    for f in files:
        print f
        name, fmat = string.split(f,'.')        
        gtname = gtdir + "\\" + f
        obname = obdir + "\\" + f
        print gtname
        print obname

        if (os.path.isfile(gtname)& (fmat == 'xml')):    
            gtpage = reader.read(gtname)
            obpage = reader.read(obname)
            tEvaluator = TableEvaluator(metrics, lables, gtpage, obpage)
            tEvaluator.getScore()
             
    tRes.output()
                                   

if __name__ == '__main__':
    main()

